#include "asm86.h"
#include "ti86asm.inc"

.org $D748
	in a,(5)
	ld (savedport5),a
	in a,(6)
	ld (savedport6),a
	ld a,(curport)
	out (6),a
	call _runindicoff
	call _clrLCD
	res 2,(iy+$0d)
	ld bc,0                 ;we'll start where we left off.
	jr showscreen

Left:
	ld bc,-1
	jr showscreen

Right:
	ld bc,1
	jr showscreen
Up:
	ld bc,-4
	jr showscreen
Down:
	ld bc,4
	jr showscreen
Plus:
	ld bc,32
	jr showscreen
Minus:
	ld bc,-32
	jr showscreen
Star:
	ld bc,256
	jr showscreen
Slash:
	ld bc,-256
	jr showscreen
Raise:
	ld bc,4096
	jr showscreen
Clear:
	ld bc,-4096
	jr showscreen
F1:
	in a,(6)
	dec a
	and %01000111
	or $40
	out (6),a
	jr showscreen
F2:
	in a,(6)
	inc a
	and %01000111
	or $40
	out (6),a
	jr showscreen
Alpha:
	ld a,(alphabit)
	xor 1
	ld (alphabit),a
	cp 0
	call z,_clrLCD
	jr showscreen
Graph:
	ld a,(graphbit)
	xor 1
	ld (graphbit),a
	cp 0
	call z,_clrLCD
	jr showscreen
Increment:
	ld hl,(caddr)
	inc (hl)
	jr showscreen
Decrement:
	ld hl,(caddr)
	dec (hl)
	jr showscreen

showscreen:
	ld hl,(caddr)
	add hl,bc
	ld (caddr),hl
	ld a,(graphbit)
	or a
	jr nz,graphscreen
	ld hl,0
	ld (_curRow),hl         ;reset cursor position
	ld de,(caddr)
	ld b,7                  ;number of rows
RowDisp:
	push bc
	ld a,d
	call Display2Char
	ld a,e
	call Display2Char           ; dump the current address
	ld hl,_curCol
	inc (hl)                ;fastest way to increment cursor column?
	ld b,4
FourByteDump:
	ld a,(de)
	call Display2Char           ;and we dump next 4 bytes
	inc de
	djnz FourByteDump
	ld a,(alphabit)
	or a
	jr z,noalpha            ;should we dump alphabits too?
	ld hl,_curCol
	inc (hl)
	ld hl,-4
	add hl,de
	ld b,4
Alphaloop:
	push bc
	ld a,(hl)
	inc hl
	cp $d6
	jr nz,SkipColon            ;we can't display the CarriageReturn (CR)
	ld a,':'
SkipColon:
	push hl
	call _putc
	pop hl
	pop bc
	djnz Alphaloop
noalpha:
	call 4A5Fh				;_newline
	pop bc
	djnz RowDisp            ;do we dare display another, we only need 7

;And now for the really easy stuff!!!
	ld hl,$3933
	ld (_penCol),hl
	ld hl,Author
	call _vputs             ;Text(57,51,"Hexedit by Tim Adkisson"
	ld a,$00
	ld (_penCol),a
	ld hl,RAMtxt
	call _vputs
	in a,(6)
	and 7
	call Get1Char
	call _vputmap
	jr keyloop

graphscreen:
	ld bc,1024
	ld de,$fc00
	ldir

keyloop:
    call GET_KEY
    or a
    jr z,keyloop
	add a,a
	ld e,a
	ld d,0
	ld hl,KeyTable
	add hl,de
	call 4010h					;_ldhlind = load (hl) into hl
	ld (SubCall),hl
	ld bc,0
	jp $0000
SubCall = $-2

More:
	call InputKeyLoop
	push af
	call Get1Char
	ld hl,$3920
	ld (_penCol),hl
	call _vputmap
	pop af
	add a,a
	add a,a
	add a,a
	add a,a
	ld b,a
	call InputKeyLoop
	ld c,a
	add a,b
	ld hl,(caddr)
	ld (hl),a
	ld a,c
	call Get1Char
	call _vputmap
InputRet:
	ld bc,0
	jp showscreen
InputKeyLoop:
	push bc
InKLoop:
	call GET_KEY
	or a
	jr z,InKLoop
	pop bc
	cp K_RAISE
	jr c,InKLoopRet
	cp K_LOG+1
	jr nc,InKLoopRet
	sub K_RAISE
	ld e,a
	ld d,0
	ld hl,translate
	add hl,de
	ld a,(hl)
	cp $ff
	ret nz
InKLoopRet:
	pop bc
	jr InputRet

Second:
	call _clrLCD
	ld hl,$1c20
	ld (_penCol),hl
	ld hl,GotoTxt
	call _vputs
SecondLoop:
	call InputKeyLoop
	ld b,a
	call Get1Char
	call _vputmap
	ld a,b
	add a,a
	add a,a
	add a,a
	add a,a
	ld b,a
	call InputKeyLoop
	ld c,a
	add a,b
	push af
	ld a,c
	call Get1Char
	call _vputmap
	pop bc			;b is now first byte of address
	call InputKeyLoop
	ld c,a
	push bc
	call Get1Char
	call _vputmap
	pop bc
	ld a,c
	add a,a
	add a,a
	add a,a
	add a,a
	ld c,a
	call InputKeyLoop
	ld h,b
	ld b,a
	add a,c
	ld l,a
	ld a,b	
	push hl
	call Get1Char
	call _vputmap
	pop hl
	ld (caddr),hl
	call _clrLCD
	jr InputRet

novars          = $CFAB   ; Number of vars found on calc
top             = $CFAC   ; Top of current page
curvar          = $CFAD   ; Current var choice
vars            = $CFAE   ; Pointers to different vars

Choosevar:
	in a,(6)
	ld (curport),a
	ld a,$47
	out (6),a
ChoosevarNoSave:
	ld hl,0
	ld (novars),hl
	ld (novars+2),hl
	ld hl,$BFFF           ; Prepare scanning variables
	ld bc,vars
Searchvars:
	ld de,($D298)
	inc de
	or a						;clear carry flag
	sbc hl,de
	jr c,ShowVarPage			;if hl>de
	add hl, de
	ex de,hl
	ld h,b
	ld l,c
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	push hl
	ld hl,novars
	inc (hl)
	ex de,hl
;by this point, (hl)=first byte of var info in vat
	dec hl
	dec hl
	dec hl
	dec hl
	dec hl
	ld b,(hl)
	dec hl
SkipName:
	dec hl
	djnz SkipName
	pop bc
	jr Searchvars
ShowVarPage:
	ld a,(novars)
	ld b,8
	cp b
	jr nc,SkipNewSize
	ld b,a
SkipNewSize:
	push bc
	call _clrLCD
	pop bc
	ld (_curRow),de			; de gets cleared by _clrLCD
	ld a,(top)
	ld e,a
	ld hl,vars
	add hl,de
	add hl,de				;hl is a pointer to table entry for top of page

ShowNames:
	push bc
	push hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl				;hl is a pointer on RAM page 7, de = old pointer
	ld a,$47
	out (6),a
	ld de,-5
	add hl,de				;hl is a pointer to size byte
	ld b,(hl)
NameDispLoop:
	push bc
	dec hl
	ld a,(hl)
	push hl
	call _putc
	pop hl
	pop bc
	djnz NameDispLoop
	call 4A5Fh			;_newLine
	pop hl
	inc hl
	inc hl
	pop bc
	djnz ShowNames
	call InvertItem
VarKeyLoop:
    call GET_KEY
    or a
    jr z,VarKeyLoop
	cp K_DOWN
	jr z,VarDown
	cp K_UP
	jr z,VarUp
	cp K_ENTER
	jr z,VarEnter
	cp K_PRGM
VarExitz1:
	jp z,VarExit
	cp K_EXIT
	jr z,VarExitz1
	cp K_DEL
	jp z,DeleteItem
	jr VarKeyLoop
VarDown:
	ld a,(curvar)
	ld b,a
	ld a,(top)
	add a,b
	inc a
	ld b,a
	ld a,(novars)
	cp b					;a=novars b=curitem
	jr z,VarKeyLoop			;end of list reached?
	ld a,(curvar)
	cp 7
	jr z,IncreaseTop		;at page end and need to scroll?
	call InvertItem
	inc a
	and 7
	ld (curvar),a
	call InvertItem
	jr VarKeyLoop
IncreaseTop:
	ld hl,top
	inc (hl)
ShowVarPage1:
	jp ShowVarPage

VarUp:
	ld a,(curvar)
	ld b,a
	ld a,(top)
	add a,b
	cp 0
	jr z,VarKeyLoop			;beginning of list reached?
	ld a,(curvar)
	cp 0
	jr z,DecreaseTop		;at page end and need to scroll?
	call InvertItem
	dec a
	and 7
	ld (curvar),a
	call InvertItem
	jr VarKeyLoop
DecreaseTop:
	ld hl,top
	dec (hl)
ShowVarPage2:
	jr ShowVarPage1

VarEnter:
	ld a,$47
	out (6),a
	call _clrLCD
	ld hl,$0002
	ld (_curRow),hl
	ld hl,VarType
	call _puts
	ld a,(curvar)
	ld b,a
	ld a,(top)
	add a,b
	ld e,a
	ld d,0
	ld hl,vars
	add hl,de
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	ld a,(hl)
	call Display2Char
	push hl
	call 4A5Fh			;_newLine
	ld hl,VarPage
	call _puts
	pop hl
	dec hl
	ld e,(hl)
	dec hl
	ld d,(hl)
	dec hl
	ld a,(hl)
	ex de,hl			;ahl=address in TI format, de=pointer in VAT
	ld b,h
	rl b
	rl a
	rl b
	rl a
	sub 3				;a is now a pointer to the page of data start
	ld (freespace),a
	ld b,a
	ld a,h
	and $3f
	or $80
	ld h,a
	ld a,b
	ld (freespace+1),hl
	push hl
	call Display2Char
	call 4A5Fh			;_newLine
	ld hl,VarStartAddr
	call _puts
	pop hl
	ld a,h
	call Display2Char	
	ld a,l
	call Display2Char	
WaitVarExit:
	call GET_KEY
	cp K_ENTER
	jr z,Jump2VarAddr
	cp K_EXIT
	jr nz,WaitVarExit
ShowVarPage3:
	jr ShowVarPage2
VarExit:
	ld a,(curport)
	out (6),a
	call _clrLCD
	ld bc,0
	jp showscreen
Jump2VarAddr:
	ld a,(freespace)
	or $40
	ld (curport),a
	ld hl,(freespace+1)
	ld (caddr),hl
	jr VarExit
DeleteItem:
	call _clrLCD
	ld hl,$1a29
	ld (_penCol),hl
	ld hl,DeleteTxt
	call _vputs
DeleteKLoop:
	call GET_KEY
	or a
	jr z, DeleteKLoop
	cp K_0
	jr nz, ShowVarPage3
	ld a,$47
	out (6),a
	ld a,(curvar)
	ld b,a
	ld a,(top)
	add a,b
	ld e,a
	ld d,0
	ld hl,vars
	add hl,de
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	ld de,_OP1+1
	dec hl
	dec hl
	dec hl
	dec hl
	dec hl					; HL-> pointer to name length
	ld b,9
DeleteCopy:
	ldi
	dec hl
	dec hl
	djnz DeleteCopy
	call _FINDSYM
	jr c, ShowVarPage3
	call _delvar
	jp ChoosevarNoSave





Exit:
	in a,(6)
	ld (curport),a
	res 0,(iy+$03)          ; Clear the ON-pressed flag
	set 2,(iy+$0d)
	call _clrScrn           ; Clear screen and the textshadow
	call _homeup
	ld hl,ProgName
	rst 20h
	rst 10h
	jr c,ResetPages
	ex de,hl
	ld a,b
	ld de,StartCopy-$D748+4
	add hl,de
	adc a,0
	ld de,StartCopy
	ld b,EndCopy-StartCopy
ExitCopy:
	push af
	push hl
	call $46C3				;inc ahl
	ld a,(de)
	ld (hl),a
	pop hl
	pop af
	call $4637				;Conv addr
	inc de
	djnz ExitCopy
ResetPages:
	ld a,(savedport6)
	out (6),a
	ld a,(savedport5)
	out (5),a
	ret

InvertItem:
	push af
	push bc
	push hl
	ld l,0
	ld a,(curvar)
	and 7
	rra
	rr l
	add a,$fc
	ld h,a
	ld b,128
InvertLoop:
	ld a,(hl)
	xor $ff
	ld (hl),a
	inc hl
	djnz InvertLoop
	pop hl
	pop bc
	pop af
	ret

Display2Char:
	push bc
	push hl
	push de
	call Get2Char
	ld a,b
	call _putc
	ld a,c
	call _putc
	pop de
	pop hl
	pop bc
	ret
Get2Char:					;very fast & efficient hex display of a
	ld c,a
	srl a
	srl a
	srl a
	srl a
	CALL Get1Char
	ld b,a
	ld a,c
	CALL Get1Char
	ld c,a
	ret
Get1Char:					;and of course for mobility, it's half-display counterpart.
	and $f
	cp $A					;equivalent to 10, the hex
	jr nc,Upper				;I'm considering whether jr would be faster
RetUpper:
	add a,$30
	ret
Upper:
	add a,$27				;the difference between lower and upper
	jr RetUpper

StartCopy:
caddr:		.db 0,0
alphabit:	.db 0
graphbit:	.db 0
savedport5:	.db 0
savedport6:	.db 0
curport:	.db 0
EndCopy:
freespace:	.db 0,0,0

ProgName = $-1
 .db 7,"hexedit"

Author:
    .db "Hexedit by Tim Adkisson",0
RAMtxt:
	.db "RAM:",0

GotoTxt:
	.db "Goto Address: ",0

VarType:
	.db "Type    ",0
VarPage:
	.db "Page    ",0
VarStartAddr:
	.db "Address ",0
DeleteTxt:
	.db "Delete? (y/n)",0

translate:			;this is really a keymap, but it cuts off before and after
.db                     $0e,$ff,$ff
.db $ff,$03,$06,$09,$ff,$0d,$ff,$ff
.db $ff,$02,$05,$08,$ff,$0c,$ff,$ff
.db $00,$01,$04,$07,$ff,$0b,$ff,$ff
.db $ff,$ff,$ff,$ff,$0f,$0a

xx = keyloop
KeyTable = $-2
.dw Down,Left,Right,  Up,xx,xx,xx,xx
.dw xx,Plus,Minus,Star,Slash,Raise,Clear,xx
.dw xx,xx,xx,xx,Increment,xx,xx,xx
.dw xx,xx,xx,xx,Decrement,xx,Choosevar,xx
.dw xx,xx,xx,xx,xx,xx,xx,xx
.dw xx,xx,xx,xx,xx,xx,Graph,Alpha
.dw xx,xx,xx,F2,F1,Second,Exit,More

;And guess what!

;the
.end
